/* $Id: audio.css,v 1.5 2006/08/21 18:13:58 drewish Exp $ */

div.audio-node {
  overflow: hidden;
}

ul.audio {
  list-style-type: none;
  margin: 0;
  padding: 0;
}

ul.audio li {
  float: left;
  margin: 1em;
}

ul.audio-info {
  list-style-type: none;
  margin: 0;
  padding: 0;
}

ul.audio-info li {
  margin: 2px 0;
  padding-left: 5px;
}

div.audio-image {
  text-align: center;
  float: right;
  padding: 5px;
  margin-left: 10px;
  background-color: #ccc;
}

div.audio-image img {
  border: 1px solid #999;
}

